﻿using ClosedXML.Excel;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Data;
using System.Windows.Forms;
using System.Diagnostics;
using System.IO;
using DocumentFormat.OpenXml.EMMA;
using DocumentFormat.OpenXml.Vml;

namespace kintone_viewer2024
{
    public partial class Form1 : Form
    {

        public string UserName { get; set; }
        public string PassWord { get; set; }

        JObject objSheetG = new JObject();


        public Form1()
        {
            InitializeComponent();
        }


        private object GetTableValue(JObject record, string fieldCode)
        {

            if (record[fieldCode]["type"].ToString() == "MODIFIER" || record[fieldCode]["type"].ToString() == "CREATOR")
            {
                return record[fieldCode]["value"]["name"];
            }
            else if (record[fieldCode]["type"].ToString() == "FILE")
            {

                string val = "";
                foreach (var fname in record[fieldCode]["value"])
                {
                    val += (string)fname["name"] + ", ";
                }
                return val;
            }
            else
            {
                if ((string)record[fieldCode]["value"] == "")
                {
                    return null;
                }
                else
                {
                    return record[fieldCode]["value"];
                }
            }
        }


        private void DistGrid()
        {
            var objConf = objSheetG[comboBox1.SelectedItem.ToString()];

            kintone_lib kintone = new kintone_lib();
            kintone.domain = (string)objConf["domain"];
            kintone.space_id = (string)objConf["space_id"];
            kintone.app_id = (string)objConf["app_id"];
            kintone.query = (string)objConf["query"];
            kintone.user_id = UserName;
            kintone.password = PassWord;

            var fields = (JArray)objConf["fields"];

            var objFields = kintone.GetFieldDef();
            if (objFields == null)
            {
                return;
            }

            string str_fields = "";
            foreach (var item in objFields.Properties())
            {
                if (item.Name != "ステータス" && item.Name != "作業者" && item.Name != "カテゴリー")
                {
                    str_fields += $"{item.Name}\t";
                }
            }
            //Debug.WriteLine(str_fields);
            File.WriteAllText("fields2clipboard.txt", str_fields);


            dataGridView1.SuspendLayout();  //描画停止

            DataTable table = new DataTable();

            foreach (string fld in fields)
            {
                if ((string)objFields[fld]["type"] == "NUMBER")
                {
                    table.Columns.Add(fld, typeof(double));
                }
                else if ((string)objFields[fld]["type"] == "DATETIME")
                {
                    table.Columns.Add(fld, typeof(DateTime));
                }
                else
                {
                    table.Columns.Add(fld, typeof(string));
                }
            }

            dataGridView1.DataSource = table;
            var records = kintone.GetAllRecords();
            File.WriteAllText("records.json", records.ToString());

            foreach (JObject item in records)
            {
                List<object> listObj = new List<object>();
                foreach (string fld in fields)
                {
                    listObj.Add(GetTableValue(item, fld));
                }
                table.Rows.Add(listObj.ToArray());
            }

            dataGridView1.ResumeLayout();   //描画再開
        }


        /// <summary>
        /// コンフィグレーション辞書を読み込む
        /// </summary>
        private JObject ReadConf()
        {
            JObject objSheet = new JObject();

            using (var wb = new XLWorkbook(@"conf\kintone_def.xlsx"))
            {
                comboBox1.Items.Clear();
                foreach (var ws in wb.Worksheets)
                {
                    JObject obj = new JObject();

                    var colCount = ws.LastColumnUsed().ColumnNumber();

                    obj["domain"] = ws.Cell(1, 2).Value.ToString();
                    obj["space_id"] = ws.Cell(2, 2).Value.ToString();
                    obj["app_id"] = ws.Cell(3, 2).Value.ToString();
                    obj["query"] = ws.Cell(5, 2).Value.ToString();

                    JArray arrFields = new JArray();
                    for (int col_index = 2; col_index <= colCount; col_index++)
                    {
                        string field = ws.Cell(4, col_index).Value.ToString();
                        arrFields.Add(field);

                    }
                    obj["fields"] = arrFields;

                    objSheet[ws.Name] = obj;
                }
            }

            return objSheet;
        }


        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
        }


        private void Form1_Load(object sender, EventArgs e)
        {

            LoginForm loginForm = new LoginForm(this);

            if (loginForm.ShowDialog() == DialogResult.OK)
            {
                this.Show();
            }
            else
            {
                MessageBox.Show("ログインに失敗しました");
                this.Close();
            }

            objSheetG = ReadConf();
            foreach (var wsName in objSheetG)
            {
                comboBox1.Items.Add(wsName.Key);
            }

        }


        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
        }


        private void button1_Click(object sender, EventArgs e)
        {
            DistGrid();
        }

        private void button2_Click(object sender, EventArgs e)
        {

            kintone_lib kintone = new kintone_lib();
            kintone.domain = "tox1i0lgjy5s";
            kintone.space_id = "";
            kintone.user_id = UserName;
            kintone.password = PassWord;

       

            kintone.GetApps();



        }
    }
}
