﻿using Newtonsoft.Json.Linq;
using System;
using System.IO;
using System.Net.Http;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;


namespace kintone_viewer2024
{
    internal class kintone_lib
    {
        private static NLog.Logger logger = NLog.LogManager.GetCurrentClassLogger();

        public string domain { get; set; }
        public string space_id { get; set; }
        public string app_id { get; set; }
        public string user_id { get; set; }
        public string password { get; set; }
        public string query { get; set; }

        JObject objFields = new JObject();



        /// <summary>
        /// 
        /// </summary>
        /// <param name="sub_url"></param>
        /// <returns></returns>
        public JObject recOperation(string sub_url)
        {
            string Authorization = Convert.ToBase64String(Encoding.UTF8.GetBytes(user_id + ":" + password));

            var client = new HttpClient();
            client.DefaultRequestHeaders.Add("ContentType", "application/json");
            client.DefaultRequestHeaders.Add(@"X-Cybozu-Authorization", Authorization);

            string guest_path = "";
            if (space_id != "")
            {
                guest_path = $"guest/{space_id}/";
            }

            string url = $"https://{domain}.cybozu.com/k/{guest_path}v1/{sub_url}";
            var response = client.GetAsync(url).Result;
            string result = response.Content.ReadAsStringAsync().Result;
            var objResult = JObject.Parse(result);
            if (response.StatusCode != System.Net.HttpStatusCode.OK)
            {
                MessageBox.Show("kintone接続エラー");
                return null;
            }

            logger.Debug(response.StatusCode);
            logger.Debug(objResult);

            return objResult;
        }


        /// <summary>
        /// 
        /// </summary>
        public JObject GetFieldDef()
        {
            var ret = recOperation($"app/form/fields.json?app={app_id}");
            if (ret == null)
            {
                return null;
            }

            objFields = (JObject)ret["properties"];
            File.WriteAllText("fields.json", objFields.ToString());

            return objFields;
        }


        /// <summary>
        /// kintoneのレコードを再帰的に全件取得する
        /// </summary>
        public JArray GetAllRecords()
        {
            JArray arrRecords = new JArray();
            int pageSize = 500;
            int offset = 0;
            int totalCount = 0;

            do
            {
                string filterCond = $"{query} limit {pageSize} offset {offset}";
                string sub_url = $"records.json?app={app_id}&totalCount=true&query={filterCond}";
                var objResult = recOperation(sub_url);
                totalCount = (int)objResult["totalCount"];
                var records = (JArray)objResult["records"];
                arrRecords.Merge(records);
                offset += pageSize; // 次のページを取得するためにoffsetを更新
            } while (offset < totalCount);

            return arrRecords;
        }


        /// <summary>
        /// 
        /// </summary>
        public object GetApps()
        {
            var ret = recOperation($"apps.json");
            if (ret == null)
            {
                return null;
            }

            return (JArray)ret["apps"];
        }
    }
}
